==> semantik_0.6.8.orig.tar.gz: semantik-0.6.8.orig/src/fig/Izltb.cpp <==
/// Copyright Thomas Nagy 2007
/// License: QPL

#include <QApplication>
#include <QPainter>
#include <QtDebug>
#include "Izltb.h"
#include "Flultb.h"
#include "Rsltb.h"

#define pad 25

izltb::izltb(dbz* felvnl) : QGraphicsRectItem()
<%
	kitzjl = 0;
	/*setBrush(QColor(0, 255, 0, 160));
	setPen(QColor(Qt::black));*/

	jlzpnt = felvnl->ellqts();

        QPen mrtdt;
        mrtdt.setWidth(1);
	mrtdt.setColor(QColor(Qt::black));
        setPen(mrtdt);

	setBrush(QColor(Qt::black));

	felvnl->scene()->addItem(this);
	setZValue(60);

	elodez = 0;
	alkt = 0;

	elktzk = 0;
	targy = 0;
	jlnrdz = felvnl;

	eler = 0;
%>

izltb::~izltb()
<%

%>

void izltb::paint(QPainter *adelk, const QStyleOptionGraphicsItem *option, QWidget * irnyjl)
<%

	adelk->setPen(pen());
	adelk->setBrush(brush());

	for (int i=0; i<lnmtat.size() - 1; ++i)
	<%
		QLineF klsirz(lnmtat[i].x(), lnmtat[i].y(), lnmtat[i+1].x(), lnmtat[i+1].y());
		adelk->drawLine(klsirz);
	%>
	QPen mrtdt = pen();
	mrtdt.setStyle(Qt::SolidLine);
	adelk->setPen(mrtdt);

	megkap(adelk, alkt, lnmtat[lnmtat.size()-1]);

	mrtdt.setWidth(1);
	adelk->setPen(mrtdt);

	QPointF prmz(3, 3);
	if (!elktzk and targy)
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = lnmtat[lnmtat.size()-1];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		rsltb *prl = 0;
		foreach (QGraphicsItem *fldlgz, scene()->items(jlnrdz->kitzjl))
		<%
			if (fldlgz->type() == gratype(37))
			<%
				prl = (rsltb*) fldlgz;
				break;
			%>
		%>
		if (prl)
		<%
			klsirz = lnmtat[0];
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
	else if (!targy and elktzk)
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = lnmtat[0];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		rsltb *prl = 0;
		foreach (QGraphicsItem *fldlgz, scene()->items(jlnrdz->kitzjl))
		<%
			if (fldlgz->type() == gratype(37))
			<%
				prl = (rsltb*) fldlgz;
				break;
			%>
		%>
		if (prl)
		<%
			klsirz = lnmtat[lnmtat.size()-1];
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
	else if (targy and elktzk and elktzk->kidlgz->hzzqjt.contains(this))
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = lnmtat[0];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		klsirz = lnmtat[lnmtat.size()-1];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));

		for (int i=1; i<lnmtat.size() - 2; ++i)
		<%
			QPointF klsirz((lnmtat[i].x() + lnmtat[i+1].x())/2, (lnmtat[i].y() + lnmtat[i+1].y())/2);
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
%>

#define xw 3.
#define yw 8.

void izltb::megkap(QPainter *adelk, int fnqll, QPointF hsztzk)
<%
	QPolygonF mrtdt;
	mrtdt<<hsztzk;
	switch (fnqll)
	<%
		case 0:
			<%
				mrtdt<<hsztzk+QPointF(-xw, -yw);
				mrtdt<<hsztzk+QPointF( xw, -yw);
			%>
			break;
		case 1:
			<%
				mrtdt<<hsztzk+QPointF(-yw, -xw);
				mrtdt<<hsztzk+QPointF(-yw,  xw);
			%>
			break;
		case 2:
			<%
                                mrtdt<<hsztzk+QPointF(-xw, yw);
                                mrtdt<<hsztzk+QPointF( xw, yw);
                        %>
			break;
		case 3:
			<%
                                mrtdt<<hsztzk+QPointF(yw, -xw);
                                mrtdt<<hsztzk+QPointF(yw,  xw);
                        %>
			break;
		default:
			break;
	%>
	adelk->drawPolygon(mrtdt);
%>

extern "C"
<%
int cel(int pos);
int akdaly(int pos);
int klsirz();
void dolog(int ax1, int ax2, int ay1, int ay2, int ap, int bx1, int bx2, int by1, int by2, int bp);
%>

int izltb::tqrost(const QRectF & mgmnkl, int fnqll, const QPointF & hsztzk)
<%
	switch (fnqll)
	<%
		case 0:
			return (hsztzk.y() > 0) ? 0 : 2;
			break;
		case 1:
			return (hsztzk.x() < 0) ? 3 : 1;
			break;
		case 2:
			return (hsztzk.y() < mgmnkl.height()) ? 2 : 0;
			break;
		case 3:
			return (hsztzk.x() > mgmnkl.width()) ? 1 : 3;
			break;
		default:
			break;
	%>
	return 2;
%>

int izltb::hzzqjt(const QRectF & mgmnkl, const QPointF & hsztzk)
<%
	QPointF klsirz = hsztzk + QPointF(mgmnkl.width()/2, mgmnkl.height()/2);
	double prmz = klsirz.x() * mgmnkl.height();
	double mrtdt = klsirz.y() * mgmnkl.width();
	if (qAbs(prmz) > qAbs(mrtdt))
	<%
		return (prmz > 0) ? 1 : 3;
	%>
	else
	<%
		return (mrtdt > 0) ? 0 : 2;
	%>
	return 0;
%>

void izltb::vmrgem()
<%

	QRectF prmz, bprl;
	QPointF mrtdt = jlnrdz->kitzjl;

	rsltb *prl = 0;
        foreach (QGraphicsItem *fldlgz, scene()->items(jlnrdz->kitzjl))
        <%
                if (fldlgz->type() == gratype(37))
                <%
                        prl = (rsltb*) fldlgz;
			break;
                %>
        %>

	if (elktzk)
	<%
		prmz = elktzk->rect().translated(elktzk->pos());
	%>
	else if (prl)
	<%
		prmz = prl->rect().translated(prl->pos());
		elodez = hzzqjt(prl->rect(), prl->pos() - mrtdt);
	%>

	if (targy)
	<%
		bprl = targy->rect().translated(targy->pos());
	%>
	else if (prl)
	<%
		bprl = prl->rect().translated(prl->pos());
		alkt = hzzqjt(prl->rect(), prl->pos() - mrtdt);
	%>

	if (!prl and (!elktzk or !targy))
	<%
		QRectF klsirz = QRectF(mrtdt - QPointF(1, 1), mrtdt + QPointF(1, 1));

		if (!elktzk)
		<%
			if (!prl) elodez = tqrost(bprl, alkt, mrtdt - targy->pos());
			prmz = klsirz;
		%>
		if (!targy)
		<%
			if (!prl) alkt = tqrost(prmz, elodez, mrtdt - elktzk->pos());
			bprl = klsirz;
		%>
	%>

	dolog(
		(int) prmz.x(), (int) (prmz.x()+prmz.width()),
		(int) prmz.y(), (int) (prmz.y()+prmz.height()),
		elodez,
		(int) bprl.x(), (int) (bprl.x()+bprl.width()),
		(int) bprl.y(), (int) (bprl.y()+bprl.height()),
		alkt);

	int ret = klsirz();
	while (kelt.size() > ret)
		kelt.takeFirst();
	while (kelt.size() < ret)
		kelt<<QPoint();

	for (int i=0; i<ret; ++i)
	<%
		int xx = cel(i);
		int yy = akdaly(i);
		kelt[i].setX(xx);
		kelt[i].setY(yy);
	%>

	nybnvn();

%>

void izltb::nybnvn()
<%
	int ret = kelt.size();

	if (kket.size() != ret - 2)
	<%
		kket.clear();
		for (int i=1; i<ret-1; ++i)
		<%
			kket.append(QPoint(0, 0));
		%>
	%>

	if (lnmtat.size() != ret)
	<%
		lnmtat.clear();
		for (int i=0; i<ret; ++i) lnmtat.append(kelt[i]);
	%>
	else
	<%
		for (int i=0; i<ret; ++i) lnmtat[i] = kelt[i];
	%>

	for (int i=0; i<kket.size(); ++i)
	<%
		int tmp = 0;
		if (kelt[i+1].x() == kelt[i+2].x())
		<%
			tmp = kket[i].x() + kelt[i+1].x();
			lnmtat[i+1].setX(ff2(tmp));
			tmp = kket[i].x() + kelt[i+2].x();
			lnmtat[i+2].setX(ff2(tmp));
		%>
		else
		<%
			tmp = kket[i].y() + kelt[i+1].y();
			lnmtat[i+1].setY(ff2(tmp));
			tmp = kket[i].y() + kelt[i+2].y();
			lnmtat[i+2].setY(ff2(tmp));
		%>
	%>

	int mx1, mx2, my1, my2;
	mx1 = mx2 = lnmtat[0].x();
	my1 = my2 = lnmtat[0].y();

	for (int i=1; i<ret; ++i)
	<%
		int nx = lnmtat[i].x(), ny = lnmtat[i].y();
		mx1 = qMin(mx1, nx);
		mx2 = qMax(mx2, nx);
		my1 = qMin(my1, ny);
		my2 = qMax(my2, ny);
	%>

	mx1 -= ff3;
	mx2 += ff3;
	my1 -= ff3;
	my2 += ff3;

	setRect(QRectF(mx1, my1, qAbs(mx2 - mx1), qAbs(my2 - my1)));
%>

bool izltb::contains(const QPointF& hsztzk) CONST
<%
	if (lnmtat.size() >= 1)
	<%
		for (int i=0; i<lnmtat.size() - 1; ++i)
		<%
			int x1 = qMin(lnmtat[i].x(), lnmtat[i+1].x());
			int x2 = qMax(lnmtat[i].x(), lnmtat[i+1].x());
			int y1 = qMin(lnmtat[i].y(), lnmtat[i+1].y());
			int y2 = qMax(lnmtat[i].y(), lnmtat[i+1].y());

			if (lnmtat[i].x() == lnmtat[i+1].x())
			<%
				if (hsztzk.x() >= x1 - 5 and hsztzk.x() <= x2 + 5)
					if (hsztzk.y() >= y1 and hsztzk.y() <= y2)
						return 1;
			%>
			else 
			<%
				if (hsztzk.y() >= y1 - 5 and hsztzk.y() <= y2 + 5)
					if (hsztzk.x() >= x1 and hsztzk.x() <= x2)
						return 1;
			%>
		%>
		return 0;
	%>
	return QGraphicsRectItem::contains(hsztzk);
%>

==> semantik_0.7.2.orig.tar.gz: semantik-0.7.2/src/fig/Izltb.cpp <==
/// Copyright Thomas Nagy 2007
/// License: QPL

#include <QApplication>
#include <QPainter>
#include <QtDebug>
#include "Izltb.h"
#include "Flultb.h"
#include "Rsltb.h"

#define pad 25

izltb::izltb(dbz* felvnl) : QGraphicsRectItem()
<%
	kitzjl = 0;
	/*setBrush(QColor(0, 255, 0, 160));
	setPen(QColor(Qt::black));*/

	jlzpnt = felvnl->ellqts();

        QPen mrtdt;
        mrtdt.setWidth(1);
	mrtdt.setColor(QColor(Qt::black));
        setPen(mrtdt);

	setBrush(QColor(Qt::black));

	felvnl->scene()->addItem(this);
	setZValue(60);

	elodez = 0;
	alkt = 0;

	vmrgem = 0;
	targy = 0;
	kelt = felvnl;

	eler = 0;
%>

izltb::~izltb()
<%

%>

void izltb::paint(QPainter *adelk, const QStyleOptionGraphicsItem *option, QWidget * irnyjl)
<%

	adelk->setPen(pen());
	adelk->setBrush(brush());

	for (int i=0; i<vnl.size() - 1; ++i)
	<%
		QLineF mrtdt(vnl[i].x(), vnl[i].y(), vnl[i+1].x(), vnl[i+1].y());
		adelk->drawLine(mrtdt);
	%>
	QPen mrtdt = pen();
	mrtdt.setStyle(Qt::SolidLine);
	adelk->setPen(mrtdt);

	nybnvn(adelk, alkt, vnl[vnl.size()-1]);

	mrtdt.setWidth(1);
	adelk->setPen(mrtdt);

	QPointF prmz(3, 3);
	if (!vmrgem and targy)
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = vnl[vnl.size()-1];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		rsltb *prl = 0;
		foreach (QGraphicsItem *fldlgz, scene()->items(kelt->kitzjl))
		<%
			if (fldlgz->type() == gratype(37))
			<%
				prl = (rsltb*) fldlgz;
				break;
			%>
		%>
		if (prl)
		<%
			klsirz = vnl[0];
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
	else if (!targy and vmrgem)
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = vnl[0];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		rsltb *prl = 0;
		foreach (QGraphicsItem *fldlgz, scene()->items(kelt->kitzjl))
		<%
			if (fldlgz->type() == gratype(37))
			<%
				prl = (rsltb*) fldlgz;
				break;
			%>
		%>
		if (prl)
		<%
			klsirz = vnl[vnl.size()-1];
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
	else if (targy and vmrgem and vmrgem->kidlgz->hzzqjt.contains(this))
	<%
		adelk->setBrush(QColor(255, 255, 0));
		QPointF klsirz = vnl[0];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		klsirz = vnl[vnl.size()-1];
		adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));

		for (int i=1; i<vnl.size() - 2; ++i)
		<%
			QPointF klsirz((vnl[i].x() + vnl[i+1].x())/2, (vnl[i].y() + vnl[i+1].y())/2);
			adelk->drawEllipse(QRectF(klsirz + prmz, klsirz - prmz));
		%>
	%>
%>

#define xw 3.
#define yw 8.

void izltb::nybnvn(QPainter *adelk, int megkap, QPointF hsztzk)
<%
	QPolygonF mrtdt;
	mrtdt<<hsztzk;
	switch (megkap)
	<%
		case 0:
			<%
				mrtdt<<hsztzk+QPointF(-xw, -yw);
				mrtdt<<hsztzk+QPointF( xw, -yw);
			%>
			break;
		case 1:
			<%
				mrtdt<<hsztzk+QPointF(-yw, -xw);
				mrtdt<<hsztzk+QPointF(-yw,  xw);
			%>
			break;
		case 2:
			<%
                                mrtdt<<hsztzk+QPointF(-xw, yw);
                                mrtdt<<hsztzk+QPointF( xw, yw);
                        %>
			break;
		case 3:
			<%
                                mrtdt<<hsztzk+QPointF(yw, -xw);
                                mrtdt<<hsztzk+QPointF(yw,  xw);
                        %>
			break;
		default:
			break;
	%>
	adelk->drawPolygon(mrtdt);
%>

extern "C"
<%
int cel(int pos);
int klsirz(int pos);
int nmsg();
void dolog(int ax1, int ax2, int ay1, int ay2, int ap, int bx1, int bx2, int by1, int by2, int bp);
%>

int izltb::tqrost(const QRectF & mgmnkl, int megkap, const QPointF & hsztzk)
<%
	switch (megkap)
	<%
		case 0:
			return (hsztzk.y() > 0) ? 0 : 2;
			break;
		case 1:
			return (hsztzk.x() < 0) ? 3 : 1;
			break;
		case 2:
			return (hsztzk.y() < mgmnkl.height()) ? 2 : 0;
			break;
		case 3:
			return (hsztzk.x() > mgmnkl.width()) ? 1 : 3;
			break;
		default:
			break;
	%>
	return 2;
%>

int izltb::hzzqjt(const QRectF & mgmnkl, const QPointF & hsztzk)
<%
	QPointF klsirz = hsztzk + QPointF(mgmnkl.width()/2, mgmnkl.height()/2);
	double prmz = klsirz.x() * mgmnkl.height();
	double mrtdt = klsirz.y() * mgmnkl.width();
	if (qAbs(prmz) > qAbs(mrtdt))
	<%
		return (prmz > 0) ? 1 : 3;
	%>
	else
	<%
		return (mrtdt > 0) ? 0 : 2;
	%>
	return 0;
%>

void izltb::jlnrdz()
<%

	QRectF prmz, bprl;
	QPointF mrtdt = kelt->kitzjl;

	rsltb *prl = 0;
        foreach (QGraphicsItem *fldlgz, scene()->items(kelt->kitzjl))
        <%
                if (fldlgz->type() == gratype(37))
                <%
                        prl = (rsltb*) fldlgz;
			break;
                %>
        %>

	if (vmrgem)
	<%
		prmz = vmrgem->rect().translated(vmrgem->pos());
	%>
	else if (prl)
	<%
		prmz = prl->rect().translated(prl->pos());
		elodez = hzzqjt(prl->rect(), prl->pos() - mrtdt);
	%>

	if (targy)
	<%
		bprl = targy->rect().translated(targy->pos());
	%>
	else if (prl)
	<%
		bprl = prl->rect().translated(prl->pos());
		alkt = hzzqjt(prl->rect(), prl->pos() - mrtdt);
	%>

	if (!prl and (!vmrgem or !targy))
	<%
		QRectF klsirz = QRectF(mrtdt - QPointF(1, 1), mrtdt + QPointF(1, 1));

		if (!vmrgem)
		<%
			if (!prl) elodez = tqrost(bprl, alkt, mrtdt - targy->pos());
			prmz = klsirz;
		%>
		if (!targy)
		<%
			if (!prl) alkt = tqrost(prmz, elodez, mrtdt - vmrgem->pos());
			bprl = klsirz;
		%>
	%>

	dolog(
		(int) prmz.x(), (int) (prmz.x()+prmz.width()),
		(int) prmz.y(), (int) (prmz.y()+prmz.height()),
		elodez,
		(int) bprl.x(), (int) (bprl.x()+bprl.width()),
		(int) bprl.y(), (int) (bprl.y()+bprl.height()),
		alkt);

	int ret = nmsg();
	while (kidlgz.size() > ret)
		kidlgz.takeFirst();
	while (kidlgz.size() < ret)
		kidlgz<<QPoint();

	for (int i=0; i<ret; ++i)
	<%
		int xx = cel(i);
		int yy = klsirz(i);
		kidlgz[i].setX(xx);
		kidlgz[i].setY(yy);
	%>

	akdaly();

%>

void izltb::akdaly()
<%
	int ret = kidlgz.size();

	if (lnmtat.size() != ret - 2)
	<%
		lnmtat.clear();
		for (int i=1; i<ret-1; ++i)
		<%
			lnmtat.append(QPoint(0, 0));
		%>
	%>

	if (vnl.size() != ret)
	<%
		vnl.clear();
		for (int i=0; i<ret; ++i) vnl.append(kidlgz[i]);
	%>
	else
	<%
		for (int i=0; i<ret; ++i) vnl[i] = kidlgz[i];
	%>

	for (int i=0; i<lnmtat.size(); ++i)
	<%
		int tmp = 0;
		if (kidlgz[i+1].x() == kidlgz[i+2].x())
		<%
			tmp = lnmtat[i].x() + kidlgz[i+1].x();
			vnl[i+1].setX(ff2(tmp));
			tmp = lnmtat[i].x() + kidlgz[i+2].x();
			vnl[i+2].setX(ff2(tmp));
		%>
		else
		<%
			tmp = lnmtat[i].y() + kidlgz[i+1].y();
			vnl[i+1].setY(ff2(tmp));
			tmp = lnmtat[i].y() + kidlgz[i+2].y();
			vnl[i+2].setY(ff2(tmp));
		%>
	%>

	int mx1, mx2, my1, my2;
	mx1 = mx2 = vnl[0].x();
	my1 = my2 = vnl[0].y();

	for (int i=1; i<ret; ++i)
	<%
		int nx = vnl[i].x(), ny = vnl[i].y();
		mx1 = qMin(mx1, nx);
		mx2 = qMax(mx2, nx);
		my1 = qMin(my1, ny);
		my2 = qMax(my2, ny);
	%>

	mx1 -= ff3;
	mx2 += ff3;
	my1 -= ff3;
	my2 += ff3;

	setRect(QRectF(mx1, my1, qAbs(mx2 - mx1), qAbs(my2 - my1)));
%>

bool izltb::contains(const QPointF& hsztzk) CONST
<%
	if (vnl.size() >= 1)
	<%
		for (int i=0; i<vnl.size() - 1; ++i)
		<%
			int x1 = qMin(vnl[i].x(), vnl[i+1].x());
			int x2 = qMax(vnl[i].x(), vnl[i+1].x());
			int y1 = qMin(vnl[i].y(), vnl[i+1].y());
			int y2 = qMax(vnl[i].y(), vnl[i+1].y());

			if (vnl[i].x() == vnl[i+1].x())
			<%
				if (hsztzk.x() >= x1 - 5 and hsztzk.x() <= x2 + 5)
					if (hsztzk.y() >= y1 and hsztzk.y() <= y2)
						return 1;
			%>
			else 
			<%
				if (hsztzk.y() >= y1 - 5 and hsztzk.y() <= y2 + 5)
					if (hsztzk.x() >= x1 and hsztzk.x() <= x2)
						return 1;
			%>
		%>
		return 0;
	%>
	return QGraphicsRectItem::contains(hsztzk);
%>
